<?php
// This file is part of the Studyplan plugin for Moodle
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <https://www.gnu.org/licenses/>.
/**
 * Show student's studyplans in a report overview
 * @package    local_treestudyplan
 * @copyright  2023 P.M. Kuipers
 * @license    https://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once("../../config.php");

require_once($CFG->libdir.'/weblib.php');

use local_treestudyplan;
use local_treestudyplan\studyplan;
use local_treestudyplan\teachingfinder;

$systemcontext = context_system::instance();

$PAGE->set_url("/local/treestudyplan/myreport.php", array());
require_login();

$PAGE->set_pagelayout('base');
$PAGE->set_context($systemcontext);

//$teachermode = has_capability("local/treestudyplan:viewuserreports", $systemcontext);
$am_teaching = teachingfinder::is_teaching($USER->id);
$have_plans = studyplan::exist_for_user($USER->id);

if ($am_teaching) {
    $PAGE->set_title(get_string('myreport_teachermode', 'local_treestudyplan'));
    $PAGE->set_heading(get_string('myreport_teachermode', 'local_treestudyplan'));
} else {
    $PAGE->set_title(get_string('report_invited', 'local_treestudyplan', "{$USER->firstname} {$USER->lastname}"));
    $PAGE->set_heading(get_string('report_invited', 'local_treestudyplan', "{$USER->firstname} {$USER->lastname}"));
}

// Load javascripts and specific css.
$PAGE->requires->css(new moodle_url($CFG->wwwroot.'/local/treestudyplan/css/bootstrap-vue/bootstrap-vue.css'));
if ($CFG->debugdeveloper) {
    $PAGE->requires->css(new moodle_url($CFG->wwwroot.'/local/treestudyplan/css/devstyles.css'));
}
$PAGE->requires->js_call_amd('local_treestudyplan/page-myreport', 'init', [$am_teaching ? 'teaching' : 'own']);

/**
 * Shortcut function to provide translations
 *
 * @param mixed $str Translation key
 * @param null|string[] $param Parameters to pass to translation
 * @param string $plugin Location to search for translation strings
 * @return string Translation of key
 */
function t($str, $param = null, $plugin = 'local_treestudyplan') {
    print get_string($str, $plugin, $param);
}

print $OUTPUT->header();

print '<div class="m-buttonbar" style="margin-bottom: 1em; text-align: right;">';

if (get_config("local_treestudyplan","enableplansharing") && !$am_teaching) {
    print '<a class="btn btn-primary" href="invitations.php" id="manage_invites">';
    print '&nbsp;<i class="fa fa-share"></i>&nbsp;'.t('manage_invites').'</a>';
}

print "</div>";
print "<div id='root'>";
print "    <div class='vue-loader' v-show='false'>";
print "        <div class='spinner-border text-primary' role='status'>";
print "            <span class='sr-only'>Loading...</span>";
print "        </div>";
print "    </div>";
print "    <div v-cloak>";
if ($am_teaching) {
    print "        <r-report type='teaching' teachermode ></r-report>";
} else {
    print "        <r-report type='own' :userid='userid'></r-report>";

}
print "    </div>";
print "</div>";

print $OUTPUT->footer();
